#include "iostm8s103k3.h"

//      
#define VALUE_TIME_POWERDOWN  20*60

//  
#define Y0 		0
#define Y0_DDR 	PD_DDR
#define Y0_CR1 	PD_CR1
#define Y0_ON 	PD_ODR |= 1 << Y0
#define Y0_OFF 	PD_ODR &= ~(1 << Y0)

#define Y1 		2
#define Y1_DDR 	PD_DDR
#define Y1_CR1 	PD_CR1
#define Y1_ON 	PD_ODR |= 1 << Y1
#define Y1_OFF 	PD_ODR &= ~(1 << Y1)

//  
#define B0 		3
#define B0_DDR 	PD_DDR
#define B0_CR1 	PD_CR1
#define B0_ON 	PD_ODR |= 1 << B0
#define B0_OFF 	PD_ODR &= ~(1 << B0)

#define B1 		4
#define B1_DDR 	PD_DDR
#define B1_CR1 	PD_CR1
#define B1_ON 	PD_ODR |= 1 << B1
#define B1_OFF 	PD_ODR &= ~(1 << B1)

//  
#define G0 		5
#define G0_DDR 	PD_DDR
#define G0_CR1 	PD_CR1
#define G0_ON 	PD_ODR |= 1 << G0
#define G0_OFF 	PD_ODR &= ~(1 << G0)

#define G1 		6
#define G1_DDR 	PD_DDR
#define G1_CR1 	PD_CR1
#define G1_ON 	PD_ODR |= 1 << G1
#define G1_OFF 	PD_ODR &= ~(1 << G1)

#define KEY 	7
#define KEY_DDR	PD_DDR
#define KEY_CR1 PD_CR1
#define KEY_CR2	PD_CR2

#define PWM_MAX 0xFF

#define UPDATE_LED( led_pwm, led_direction ) 					\
			if (direction & (1 << (led_direction))) 			\
			{													\
				if (++(led_pwm) == PWM_MAX)						\
					direction &= ~(1 << (led_direction));		\
			}													\
			else												\
				if (--(led_pwm) == 0)							\
					direction |= 1 << (led_direction)			

enum
{
	Y0_DIRECTION = 0,
	Y1_DIRECTION,
	B0_DIRECTION,
	B1_DIRECTION,
	G0_DIRECTION,
	G1_DIRECTION
};

typedef unsigned char	u8;
typedef unsigned int	u16;

u8 y0_pwm, y1_pwm, b0_pwm, b1_pwm, g0_pwm, g1_pwm;

u8 direction;
u16 time_powerdown = 0;
u8 key_down = 0;

void main()
{
	//     16/8 = 2MHz
	CLK_CKDIVR = 3 << 3;

	// timer 1 Hz
	TIM2_CR1 |= 1 << 0;
	TIM2_IER |= 1 << 0;
	TIM2_PSCR = 0x0E;
	TIM2_ARRH = 0x00; 
	TIM2_ARRL = 0x7D;
	
	// timer for PWM
	TIM4_CR1 |= 1 << 0;
	TIM4_IER |= 1 << 0;
	TIM4_PSCR = 0x00;
	TIM4_ARR = 0x80;
	
	//  
	PA_DDR = 0xFF;
	PB_DDR = 0xFF;
	PC_DDR = 0xFF;
	PD_DDR = 0xFF;
	PE_DDR = 0xFF;
	PF_DDR = 0xFF; 	
	
	Y0_DDR |= 1 << Y0;
	Y0_CR1 |= 1 << Y0;
	Y0_OFF;
	Y1_DDR |= 1 << Y1;
	Y1_CR1 |= 1 << Y1;
	Y1_OFF;
	B0_DDR |= 1 << B0;
	B0_CR1 |= 1 << B0;
	B0_OFF;
	B1_DDR |= 1 << B1;
	B1_CR1 |= 1 << B1;
	B1_OFF;
	G0_DDR |= 1 << G0;
	G0_CR1 |= 1 << G0;
	G0_OFF;	
	G1_DDR |= 1 << G1;
	G1_CR1 |= 1 << G1;
	G1_OFF;
	
	KEY_DDR &= ~(1 << KEY);
	KEY_CR1 |= 1 << KEY;
	KEY_CR2 |= 1 << KEY;

	// interrupt enable 
	asm ("rim");
	
	asm ("halt");
	
	//   ,    
	for (u8 i =0; i < PWM_MAX; i++)
	{
		if (i > 110)
			y0_pwm++;
		if (i > 10)
			y1_pwm++;
		if (i > 40)
			b0_pwm++;
		if (i > 240)
			b1_pwm++;
		if (i > 180)
			g0_pwm++;
		if (i > 30)
			g1_pwm++;	
		for (u16 j = 0; j < 0xFF; j++)
			asm ("wfi");
	}
	
	key_down = 0;
	KEY_CR2 |= 1 << KEY;
	
	while (1)
	{
		UPDATE_LED (y0_pwm, Y0_DIRECTION);
		UPDATE_LED (y1_pwm, Y1_DIRECTION);
		UPDATE_LED (b0_pwm, B0_DIRECTION);
		UPDATE_LED (b1_pwm, B1_DIRECTION);			
		UPDATE_LED (g0_pwm, G0_DIRECTION);
		UPDATE_LED (g1_pwm, G1_DIRECTION);

		if (key_down || (time_powerdown >= VALUE_TIME_POWERDOWN))
		{
			//   
			for (u8 i =0; i < PWM_MAX; i++)
			{
				if (y0_pwm > 0)
					y0_pwm--;
				if (y1_pwm > 0)
					y1_pwm--;
				if (b0_pwm > 0)
					b0_pwm--;
				if (b1_pwm > 0)
					b1_pwm--;
				if (g0_pwm > 0)
					g0_pwm--;
				if (g1_pwm > 0)
					g1_pwm--;	
				for (u16 j = 0; j < 0xFF; j++)
					asm ("wfi");
			}
			
			// reset device
			WWDG_CR = 0;    
			WWDG_CR |= 1 << 7;
			WWDG_CR |= 1 << 6; 		
		}

		for (u16 i = 0; i < 0x3FF; i++)
			asm ("wfi");
	}
}

#pragma vector=TIM2_OVR_UIF_vector  
__interrupt void time_one_second (void)
{
	time_powerdown++;

	TIM2_SR1 &= (~(1 << 0));
}

#pragma vector = TIM4_OVR_UIF_vector  
__interrupt void pwm_body (void)
{
	static u8 time_pwm = 0;
	
	if (++time_pwm == PWM_MAX)
	{
		time_pwm = 0;
		if (y0_pwm)
			Y0_ON;
		if (y1_pwm)
			Y1_ON;
		if (b0_pwm)
			B0_ON;
		if (b1_pwm)
			B1_ON;
		if (g0_pwm)
			G0_ON;
		if (g1_pwm)
			G1_ON;
	}  
  
	if (time_pwm >= y0_pwm)
		Y0_OFF;
	if (time_pwm >= y1_pwm)
		Y1_OFF;   
	if (time_pwm >= b0_pwm)
		B0_OFF;
	if (time_pwm >= b1_pwm)
		B1_OFF;
	if (time_pwm >= g0_pwm)
		G0_OFF;
	if (time_pwm >= g1_pwm)
		G1_OFF;    
 
	TIM4_SR &= ~(1 << 0);
}

#pragma vector = 0x2  
__interrupt void key_tli (void)
{
	key_down++;
	KEY_CR2 &= ~(1 << KEY);
}